#script to process contents of mods folder in order to automatically upgrade mods
#broken by new release
#makes new folder new_mods
#reads .lua and .json and uses regexp to replace obsolete code
#################################################################################
	#READING AND IMPROVING
#As I've stated here: https://forums.factorio.com/viewtopic.php?f=137&t=26613
#this code might be a dead end, whilst lua implementation might provide more satisfying experience.

#Nevertheless, if you insist on continuing, the code down here is quite entangled,
#the idea was to have a single file for the updater, which results in this complication.
#The structure of the code is the following:
	#first there's constant string `migration`, which identifies which array of transformations to use 
	#(as of 04.07.16 there were only two values and actually '0.12.35-0.13.3' brought mods to .13 standards fully.
	#then there's fundamental part of the program: main(), it contains all the operations with the filesystem
	#it is actually called in the last line of the file.

	#After that, auxilary text processing functions are defined (brace search and such).
	#After that, the transfomation functions are defined. 
	#These functions might be of use for different migrations with different substitution patterns.
	#Thus there actually are generators for them, which take the lists of substitutions and
	#generate functions that would perform them.
	
	#Finally there's the definition of transformations. They're once again functions that are put into a 
	#table with corresponding key (e.g. '0.12.35-0.13.3') and then are called in main() to obtain the table of actions to perform
	#on text.
	#There are separate transformations for .json and .lua files.
	#In essence, a bunch of old:new code pairs are fed to different function generators from previous paragraph
	#and obtained functions(code) are put into a single table that is later traversed and called in main().
#################################################################################
from os import walk
from os.path import isdir
from os import mkdir
from shutil import copy
from string import ascii_letters as ascii
import re

migration='0.12.35-0.13.3'
#creation, copying and writing of files
def main():#will be called at the end of file
	if not isdir('./mods'):
		print('"mods" folder not found')
		exit()
	if isdir("./new_mods"):
		print('You should make "new_mods" directory disappear before running the script')
		exit()
	mkdir('./new_mods')

	print('updating')
	for (_,mods,_) in walk('mods'):
		break
	#here, we get major steps for lua updating:
	#in 12.35-13.0 for example:
	# 1. Simple renaming without regex syntax
	# 2. Renaming with regex, performs search in dictionary for every match
	# 3. Semi-manual code analysis, python regex can't express nested parenthesis
	transformations =init_expressions_for(migration)
	json_update=init_json_for(migration)
	modified=[]
	for m in mods:
		print('_____' + m + '_____')
		match=re.search(r"(.*(\d+?\.\d+?\.))(\d+?)$",m)
		if not match.group(2) or not match.group(3):
			print('modname format incorrect, skipping')
			continue
		#old version
		v=match.group(2)+match.group(3)
		#new folder name
		n=match.expand(match.group(1)+str(1+int(match.group(3))))
		#new version
		nv=match.expand(match.group(2)+str(1+int(match.group(3))))
		for (dirpath, dirnames, filenames) in walk('mods/'+m):
			path=dirpath.replace(m,n,1).replace('mods','new_mods',1)
			mkdir(path)
			for fn in filenames:
				#whether update did some replacements
				significant=False
				match=re.search(r"\.(lua)|(info\.json)",fn)
				if match:
					print('doing ',fn)
					f=open(dirpath + '/' + fn,encoding="latin-1",errors='ignore')
					code=f.read()
					f.close()
					nf=open(path+'/'+fn,'w')
					if  fn=='info.json':
						new_code=re.sub(r'[\'\"]version[\'\"]\s*?:\s*?[\'\"]([0-9\.]+)[\'\"]\s*?,?',r'"version":"'+nv+r'",',code)
						for t in json_update:#apply all specified modifications
							new_code=t(new_code)
					else:
						new_code=code
						for t in transformations:
							new_code=t(new_code)
						if not significant and new_code!=code: significant=True#check if changes made
					nf.write(new_code)
					nf.close()
				else:
					copy(dirpath+'/'+fn,path+'/')
				#end if match
				if significant: modified.append(path+'/'+fn)
			#end for fn in filenames:
		#end for (dirpath, dirnames, filenames) in walk('mods/'+m):
	#end for m in mods:
	if len(modified)>0:
		print('Following files have been modified')
		for k in modified:print(k)

#pattern definition, no system calls below this line
########################################################################
########################################################################
migrations={}
json_updates={}
def init_expressions_for(n):
	if n in migrations:
		return migrations[n]()
	else:
		print('Unknown migration specified in code')
		exit()

def init_json_for(n):
	if n in json_updates:
		return json_updates[n]()
	#else:
		#print('Unknown migration specified in code')
		#exit()
#usable funcs, used to generate transformations of file
#put their outputs in list of tuples and it'll be fed to script
def simple_renaming(simple):
	#initializator for easy processings
	#takes dictionary of exact replacements
	#returns tupple of regex obj and replace function
	string1='(?aims)('
	for k in simple:
		string1=string1+re.escape(k)+'|'
	string1=string1+')'
	simpl=re.compile(string1)
	def srep(m):
		if m.group(1)!='':
			print('\treplacing ', m.group(1), ' with ',simple[m.group(1)])
			return simple[m.group(1)]
	def transform(code):
		return re.sub(simpl,srep,code)
	return transform

def medium_replacement(medium):
	#medium difficulty processing
	#takes dictionary with regex syntax
	string2='(?aims)'
	i=1;
	mres={}
	for k,v in medium.items():
		string2=string2+'(?P<'+ascii[i]+'>'+k+')|'
		mres[ascii[i]]=v
		i=i+1;
	def mrep(m):
		for k,v in mres.items():
			if m.group(k):
				if callable(v):
					q= v(m.group(k))
				else:
					q= v
				print('\treplacing ', m.group(k), ' with ',q)
				return q
	med=re.compile(string2)
	def transform(code):
		return re.sub(med,mrep,code)
	return transform

#handy for hardest replacements below
def count_lines(string, pos):
	#returns linenumber, where pos is located
	l=1
	lin=re.compile(r'\n')
	for m in lin.finditer(string,0,pos):l=l+1
	return l
def find_bra(string,pos=0,ope='(',clo=')'):
	#finds in 'string' the outermost brakets to the right of 'pos' specified by 'ope' and 'clo'
	#returns the bracket positions (left,right)
	reg=re.compile('(?P<left>\\'+ope+')|(?P<right>\\'+clo+')')
	num=0
	first=reg.search(string,pos)
	if first and first.group('left'):
		first=first.start()
		num=num+1
	else: return None

	index=first+1
	while index < len(string) and num>0:
		m=reg.search(string,index)
		if not m: break
		if m.group('left'):
			num=num+1
		elif m.group('right'):
			if num>1: num=num-1
			else:
				return(first,m.end())
		index=m.end()
	return None
def find_out_bra(string,pos=0):
	#wrapper to find outermost of  any possible brakets
	bras={'(':')','[':']','{':'}'}
	res=()
	for k,v in bras.items():
		bp=find_bra(string,pos,k,v)
		if bp and( (not res) or res[0]>bp[0]): res=bp;
	return res
def encompassing_block(code,identifier,braces,limits=0):
	#find `identifier` and return position of `braces` that are around it
	#limits specify the are to look at in the code
	id=re.compile(identifier)
	if limits!=0:
		m=id.search(code,limits[0])
	else:
		m=id.search(code)
	if m:
		i1=m.start()
		i2=m.end()
		if limits!=0 and (i1<limits[0] or i2>limits[1]): return
		
		
		brac=re.compile(braces[0])
		i0=0
		i00=0
		while True:
			br=brac.search(code[:i1],i00)
			if (not br or br.start()>i1):break
			i00=br.end()
			bp=find_bra(code,pos=br.start(),ope=braces[0],clo=braces[1])
			if bp and bp[1]>=i2 and bp[0]<=i1:
				result=bp
		#endwhile
		
		#bp=find_bra(code,pos=i0,ope=braces[0],clo=braces[1])
		bp=result
		if bp and bp[0]<=i1 and bp[1]>=i2:
			return bp
		#endif
	#endif
#end def encompassing_block(code,identifier,braces):
def insert_in(code,identifier,tag,transform,braces,limits=0):
	#utility function that finds `identifier` in `code`, looks for enclosing braces specified by tuple `braces`
	# and if there's no `tag` within the braces calls `transform` with (code,left_brace_pos,right_brace_pos)
	#limits specify the are to look at in the code
	m=re.search(identifier,code);
	if limits!=0:
		bp=encompassing_block(code,identifier,braces,limits)
	else:
		bp=encompassing_block(code,identifier,braces)
	if bp:
		fi=re.compile(tag)
		fp=fi.search(code,bp[0],bp[1])
		if fp:
			return code
		else:
			if callable(transform):
				return transform(code,bp[0],bp[1])
			else:
				return code[:bp[0]+1]+transform+code[bp[0]+1:]
		#endif
	else:
		return code
	#endif
#end def insert_in(code,tag,new_tag,transform)

#hardest replacements are here.
#they're specific function to perform a particular transformation over text
def ins_arg(hard):
	#returns func that
	#transforms the function calls () to table indexing [] and so on
	#takes a dictionary of dicts with particular fields (see #'0.12.35-0.13.0' example)
	bras={'(':')','[':']','{':'}'}
	string3='(?aims)'
	i=1;
	hres={}
	for k in hard:
		string3=string3+'(?P<'+ascii[i]+'>'+k['old']+'\W*?\\'+k['old_bra']+')|'
		hres[ascii[i]]=k
		i=i+1;

	reg=re.compile(string3)

	def transform(code):
		#in current modification is fit for single character brakets only
		index=0#reader position
		ms=[]#modifications to perform
		lett=list(code)#will do modifications on list of letters
		while index<len(code):#scan code for names to replace
			m=reg.search(code,index)
			index=index+1
			if not m: break
			for k,v in hres.items():
				if m.group(k):
					print('\tchanging parenthis',m.group(k))
					index=m.end()-1
					r=bras[v['old_bra']]
					bp=find_bra(code,pos=index,ope=v['old_bra'],clo=r)#braket positions
					if not bp:
						print ('parenthises fail');
						print(count_lines(code,index))
						exit()#error
					ope=v['new_bra']; clo=bras[ope];#brakets are substituted in place
					lett[bp[0]]=ope
					lett[bp[1]-1]=clo
					ms.append((v,(m.start(),m.end())))#memorize what names to replace
		for i in range(len(ms)-1,-1,-1):
			v=ms[i]
			del(lett[v[1][0]:v[1][1]-1])
			lett.insert(v[1][0],v[0]['new'])
		new_code=''.join(lett)
		return new_code
	return transform

def shuffle_args(hard):
	#returns function that finds function with two args and swaps places of args
	bras={'(':')','[':']','{':'}'}
	string4='(?aims)'
	i=1;
	hres={}
	for k in hard:
		string4=string4+'(?P<'+ascii[i]+'>'+k['old']+'\W*?\\'+k['old_bra']+')|'
		hres[ascii[i]]=k
		i=i+1;
	reg=re.compile(string4)
	def shuff(code):
		index=0#reader position
		modifications=[]#modifications to perform
		lett=list(code)#will do modifications on list of letters
		comma=re.compile(',')
		while index<len(code):#scan code for names to replace
			m=reg.search(code,index)
			index=index+1
			if not m: break
			for k,v in hres.items():
				if m.group(k):
					print(m.group(k))
					print('\tshuffling arguments',m.group(k))
					index=m.end()
					l=v['old_bra']
					r=bras[l]
					bp=find_bra(code,pos=index-1,ope=l,clo=r)#braket positions
					if not bp:
						print ('parenthises fail on line: ',count_lines(code,index))
						exit()#error
					#count commas
					commas=[];
					for x in comma.finditer(code,bp[0]+1,bp[1]): commas.append(x.start());
					err=False
					if len(commas)==0: err=True
					if len(commas)>0: #ms.append((bp[0]+1,p[0]-1,bp[2]));
					#filter out commas in nested brakets
						brakets=[]
						ind=bp[0]+1
						while ind<bp[1]:
							bp2=find_out_bra(code,pos=ind)
							if not bp2 or bp2[1]>bp[1]: break
							commas=[x for x in commas if (x<bp2[0] or x>bp2[1]-1)]
							if len(commas)==0: err=True; break
							ind=bp2[1]
						if len(commas)>1: err=True
					if err: print('error, shuffle called on not two arg fun ',count_lines(code,index))#will just move on
					else:
						modifications.append((bp[0]+1,commas[0],bp[1]-1));
						print('\tshuffling arguments in ',m.group(k))
				#end if m.group(k):
			#end for k,v in hres.items():
		#end while index<len(code)
		for m in reversed(modifications):
			left=code[m[0]:m[1]]
			right=code[m[1]+1:m[2]]
			del(lett[m[1]+1:m[2]])
			lett.insert(m[1]+1,left)
			del(lett[m[0]:m[1]])
			lett.insert(m[0],right)
		new_code=''.join(lett)
		return new_code
	#end def shuff
	return shuff
#end def shuffle_args(hard)

def update_prototype(hard):#takes array of tables with fields 'prototype', which serve to identify target
	#'field' - which is a string to identify precence of needed field
	#and fields 'value' which is a text to add to prototype if field not present
	#returns function to perfom the said transformations on code
	string4='(?aims)'
	i=1;
	hres={}
	for k in hard:
		string4=string4+'(?P<'+ascii[i]+'>'+k['prototype']+')|'
		hres[ascii[i]]=k
		i=i+1;
	reg=re.compile(string4)
	brac=re.compile('{')

	def upd(code):#start full-fledged analysis
		i=0; q=1;
		#print('updating')
		#print(reg)
		#while i<len(code):
			#q=re.search('type\s*?=\s*?[\',\"]\w+-turret[\',\"]\s*?',code,i)
			#if not q:break
			#print(q)
			#print('printed q')
			#i=q.end()
		index=0#reader position
		while index<len(code):#scan code for names to replace
			m=reg.search(code,index)
			index=index+1
			if not m: break
			for k,v in hres.items():
				if m.group(k):
					i1=m.start()
					i2=m.end()
					index=m.end()
					br=brac.search(code[:i1])
					if br:
						print('\tupdating prototype',m.group(k))
						bp=encompassing_block(code,v['prototype'],('{','}'),limits=(i1,i2))
						if bp and bp[0]<i1 and bp[1]>i2:
							fi=re.compile(v['field'])
							fp=fi.search(code,bp[0],bp[1])
							if fp:
								print(fp.group(1) + 'is present already')
							else:
								print('adding' + v['value'])
								code=code[:i2]+v['value']+code[i2:]
							#endif
						#endif
					#endif
				#endif
			#endfor
		#endwhile
		return code
	# end def upd(code)
	return upd
#end def update_prototype(hard)		
	
#the actual pattern definition
def json_():
	#returns function to generate new standard json
	actions=[]
	def fu(code):
		def fi(word,code):#find word
			r=re.search(r'[\'\"]'+word+r'[\'\"]\s*?:\s*?[\'\"](.*?)[\'\"]\s*?,?',code)
			if r:
				res=r.group(1).replace("\\",'\\\\')
			else:
				res=""
			return res
		name=fi('name',code)
		version=re.search(r'[\'\"]version[\'\"]\s*?:\s*?[\'\"]([0-9\.]+)[\'\"]\s*?,?',code).group(1)
		title=fi('title',code)
		author=fi('author',code)
		contact=fi('contact',code)
		homepage=fi('homepage',code)
		description=fi('description',code)
		license=fi('license',code)
		contact=fi('contact',code)
		dependencies=re.search(r'[\'\"]dependencies[\'\"]\s*?:\s*?\[(.+)\]\s*?,?',code)
		factorio_version='0.13.'
		if not dependencies:
			dependencies=re.search(r'[\'\"]dependency[\'\"]\s*?:\s*?([\'\"].*?[\'\"])\s*?,?',code)
		if dependencies:
			dependencies=re.sub('\'','"',dependencies.group(1))
			if not re.search(r'[\'\"]dependen\w+?[\'\"]\s*?:\s*?[\'\"]base\s*?[=<>]?[^\'\"]*?[\'\"]\s*?,?',dependencies):
				dependencies=dependencies+',"base"'
		else:
			dependencies='"base"'
		return ('{\n"name":"'+name+
			'",\n"version":"'+version+
			'",\n"title":"'+title+
			'",\n"author":"'+author+
			'",\n"contact":"'+contact+
			'",\n"homepage":"'+homepage+
			'",\n"description":"'+description+
			'",\n"factorio_version":"'+factorio_version+
			'",\n"dependencies":['+dependencies+']'+
			'\n}')
	actions.append(fu)
	return actions
def _():#'0.12.35-0.13.0'
	#list of changes
	simple={
		#these substituitions are simple and even if recognized wrongly, will not break most things
		#no regex in them
		'basic-transport-belt':'transport-belt',
		'transport-belt-to-ground':'underground-belt',
		'basic-splitter':'splitter',
		'basic-inserter':'inserter',
		'basic-mining-drill':'electric-mining-drill',
		'basic_beacon':'beacon',
		'basic-bullet-magazine':'firearm-magazine',
		'piercing-bullet-magazine':'piercing-rounds-magazine',
		'basic-grenade':'grenade',
		'basic-armor':'light-armor',
		'basic-modular-armor':'modular-armor',
		'basic-laser-defense-equipment':'personal-laser-defense-equipment',
		'basic-exoskeleton-equipment':'exoskeleton-equipment',
		"basic-electric-discharge-defense-equipment": "discharge-defense-equipment",
		"basic-electric-discharge-defense-remote": "discharge-defense-remote",
		"basic-accumulator": "accumulator",
		'groupstate':'group_state',
		'circuitconnector':'circuit_connector',
		'circuitconditionindex':'circuit_condition_index',
		'trainstate':'train_state',
		'logistic_robot_storage_modifier':'logistic_robot_storage_bonus',
		'smart-container':'container',
		'smart-chest':'steel-chest',
		'smart-inserter':'filter-inserter',
		'research-effectivity-':'research-speed-',
		'basic-beacon':'beacon',
		}
	def gui_style(text):#pyhon doesnt let to define this one where it's needed so it's here
		#process substring related to gui_style.parent='default
		s=re.search("type\s*?=\s*?[\'\"](\S+?_style)[\'\"]",text)
		assert(s)
		return re.sub("parent\s*?=\s*?[\'\"](default)[\'\"]","parent=\""+s.group(1)+"\"",text)
		
	medium={
		#these are needing named groups to be replaced
		'require\W*?\(?\s*?\'defines(\.lua)*\'\s*?\)?':'',
		'require\W*?\(?\s*?\"defines(\.lua)*\"\s*?\)?':'',
		'game\.player(?!s)':'game.players[1]',
		'game\.local_player':'game.player',
		'game.regenerate_tiles\(\)':'',
		'ipairs\W*\(game.players':'pairs(game.players',
		'ipairs\W*\(\W*game.surfaces':'pairs(game.surfaces',
		'ipairs\W*\(\W*game.entity_prototypes':'pairs\(game.entity_prototypes',
		'ipairs\W*\(\W*game.item_prototypes':'pairs\(game.item_prototypes',
		'ipairs\W*\(\W*game.fluid_prototypes':'pairs\(game.fluid_prototypes',
		'ipairs\W*\(\W*force.recipes':'pairs\(force.recipes',
		'ipairs\W*\(\W*force.technologies':'pairs\(force,technologies',
		'data:extend\s*?\(?\s*?{\s*?}\s*?\)?\s*?':'',
		'(?<!electric-)(?<!burner-)?mining-drill':'electric-mining-drill',
		#really long string for new fields in prototypes
		'type\s*?=\s*?[\',\"]train-stop[\',\"]\s*?,':
			"type='train-stop',\nanimation_ticks_per_frame = 1,\ncircuit_wire_connection_points=\t{\n      {\n        shadow =\n        {\n          red = {-0.375, 1.21875},\n          green = {-0.53125, 1.21875}\n        },\n        wire =\n        {\n          red = {-0.5, 1.09375},\n          green = {-0.65625, 1.09375}\n        }\n      },\n      {\n        shadow =\n        {\n          red = {-0.875, -0.71875},\n          green = {-0.875, -0.84375},\n        },\n        wire =\n        {\n          red = {-1.0625, -0.84375},\n          green = {-1.0625, -0.96875},\n        }\n      },\n      {\n        shadow =\n        {\n          red = {0.78125, 0.46875},\n          green = {0.59375, 0.46875},\n        },\n        wire =\n        {\n          red = {0.65625, 0.34375},\n          green = {0.5, 0.34375},\n        }\n      },\n      {\n        shadow =\n        {\n          red = {-0.0625, 1.28125},\n          green = {-0.0625, 1.15625},\n        },\n        wire =\n        {\n          red = {-0.25, 1.15625},\n          green = {-0.25, 1.03125},\n        }\n      },\n    },\n    circuit_connector_sprites =\n    {\n      get_circuit_connector_sprites({0.5625-1, 1.03125}, {0.5625-1, 1.03125}, 0), --N\n      get_circuit_connector_sprites({-0.78125, 0.28125-1}, {-0.78125, 0.28125-1}, 6), --E\n      get_circuit_connector_sprites({-0.28125+1, 0.28125}, {-0.28125+1, 0.28125}, 0), --S\n      get_circuit_connector_sprites({0.03125, 0.28125+1}, {0.03125, 0.28125+1}, 6), --W\n    },\n    circuit_wire_max_distance = 7.5,",
		'type\s*?=\s*?[\',\"]storage-tank[\',\"]\s*?,':
			"type='storage-tank',\ncircuit_wire_connection_points=\t{\n      {\n        shadow =\n        {\n          red = {-0.375, 1.21875},\n          green = {-0.53125, 1.21875}\n        },\n        wire =\n        {\n          red = {-0.5, 1.09375},\n          green = {-0.65625, 1.09375}\n        }\n      },\n      {\n        shadow =\n        {\n          red = {-0.875, -0.71875},\n          green = {-0.875, -0.84375},\n        },\n        wire =\n        {\n          red = {-1.0625, -0.84375},\n          green = {-1.0625, -0.96875},\n        }\n      },\n      {\n        shadow =\n        {\n          red = {0.78125, 0.46875},\n          green = {0.59375, 0.46875},\n        },\n        wire =\n        {\n          red = {0.65625, 0.34375},\n          green = {0.5, 0.34375},\n        }\n      },\n      {\n        shadow =\n        {\n          red = {-0.0625, 1.28125},\n          green = {-0.0625, 1.15625},\n        },\n        wire =\n        {\n          red = {-0.25, 1.15625},\n          green = {-0.25, 1.03125},\n        }\n      },\n    },\n    circuit_connector_sprites =\n    {\n      get_circuit_connector_sprites({0.5625-1, 1.03125}, {0.5625-1, 1.03125}, 0), --N\n      get_circuit_connector_sprites({-0.78125, 0.28125-1}, {-0.78125, 0.28125-1}, 6), --E\n      get_circuit_connector_sprites({-0.28125+1, 0.28125}, {-0.28125+1, 0.28125}, 0), --S\n      get_circuit_connector_sprites({0.03125, 0.28125+1}, {0.03125, 0.28125+1}, 6), --W\n    },\n    circuit_wire_max_distance = 7.5,",
		
		"type\s*?=\s*?[\'\"]\S+_style[\'\"]\s*?,\s*?parent\s*?=\s*?[\'\"]default[\'\"]":gui_style,
		}
	hard=[
		#python cant find balanced parenteses will have to be done by hand
		#each of these has a potential to get personal replacer function
		{'old':'get_filter','new':'get_filter','transform':'shuffle','old_bra':'(','new_bra':'('},
		{'old':'set_filter','new':'set_filter','transform':'shuffle','old_bra':'(','new_bra':'('},
		{'old':'game.get_player','new':'game.players','transform':'insert_arg','old_bra':'(','new_bra':'['},
		{'old':'game.get_surface','new':'game.surfaces','transform':'insert_arg','old_bra':'(','new_bra':'['},
		{'old':'game.get_item_prototype','new':'game.item_prototypes','transform':'insert_arg','old_bra':'(','new_bra':'['},
	]
	harder=[#this one is so complex I can't just put it in any of the above patterns, I'll just define it below this table
	]
	def add_f_c(code):
	#add_frame_count_to_turret_animations
		j=0
		height=re.compile('height\s*?=')
		fc=re.compile('frame_count\s*?=')
		while j<len(code):#scan code for names to replace
			t=encompassing_block(code,'width\s*?=',('{','}'),limits=(j,len(code)))
			if not t: break
			i=t[0]
			j=t[1]
			if height.search(code,i,j) and not fc.search(code,i,j):
				code=code[:i+1]+'\nframe_count=1,'+code[i+1:]
				print('adding frame_count')
		#endwhile
		return code
	#end def add_frame_count_to_turret_animations	
	actions=[]
	actions.append(simple_renaming(simple))
	actions.append(medium_replacement(medium))
	h1=[]; h2=[]; h3=[];
	for k in hard:
		if k['transform']=='insert_arg':
			h1.append(k)
		elif k['transform']=='shuffle':
			h2.append(k)
	actions.append(ins_arg(h1))
	actions.append(shuffle_args(h2))
	actions.append(add_f_c)
	
	return actions

migrations['0.12.35-0.13.0']=_
json_updates['0.12.35-0.13.0']=json_

def _():#0.13.0-0.13.3
	simple={
		'logistic-robot-storage':'worker-robot-storage',
		'logistic-robot-speed':'worker-robot-speed',
	}
	hard=[
		{'prototype':'type\s*?=\s*?[\'\"]locomotive[\'\"]\s*?,','field':'reversing_power_modifier','value':'reversing_power_modifier=0.6,\nwheels={priority = "very-low",\nwidth=1,height=1,\ndirection_count=1,\nfilenames={"__core__/graphics/empty.png"},\nline_length=1,lines_per_file=1,\n},'},
		{'prototype':'type\s*?=\s*?[\'\"]cargo-wagon[\'\"]\s*?,','field':'reversing_power_modifier','value':'reversing_power_modifier=0.6,\nwheels={priority = "very-low",\nwidth=1,height=1,\ndirection_count=1,\nfilenames={"__core__/graphics/empty.png"},\nline_length=1,lines_per_file=1,\n},'},
		{'prototype':'type\s*?=\s*?[\',\"]player[\'\"]\s*?,','field':'item_pickup_distance','value':'item_pickup_distance=1,\nloot_pickup_distance=2,\n'},
		{'prototype':'type\s*?=\s*?[\'\"]\w+?-turret[\'\"]\s*?,','field':'call_for_help_radius','value':'call_for_help_radius=40,\n'},
		{'prototype':'type\s*?=\s*?[\'\"]module[\'\"]\s*?,','field':'category','value':'category = "speed",\ntier=1,\n'},
		{'prototype':'type\s*?=\s*?[\'\"]transport-belt[\'\"]\s*?,','field':'connector_frame_sprites','value':
			'''    connector_frame_sprites = transport_belt_connector_frame_sprites,
	circuit_connector_sprites = transport_belt_circuit_connector_sprites,
    circuit_wire_connection_point = transport_belt_circuit_wire_connection_point,
    circuit_wire_max_distance = transport_belt_circuit_wire_max_distance,\n'''},
		{'prototype':'type\s*?=\s*?[\'\"]offshore-pump[\'\"]\s*?,','field':'circuit_connector_sprites','value':
			'''   circuit_connector_sprites =
    {\nget_circuit_connector_sprites({0.90625, -0.15625}, nil, 0),\nget_circuit_connector_sprites({0, 0.03125}, nil, 6),
      get_circuit_connector_sprites({-0.9375, -0.25}, nil, 4),\nget_circuit_connector_sprites({0.125, -1.3125}, nil, 2),
    },\n    circuit_wire_max_distance = 7.5,\n'''},
		{'prototype':'filename = "__base__/graphics/entity/stone-furnace/stone-furnace-fire.png",[\s\S]*?shift.*?}',
		'field':',\njust_do_it,','value':''',\n          line_length = 8,
          width = 22,\n          height = 87,\n          frame_count = 48,
          axially_symmetrical = false,\n          direction_count = 1,\n          shift = {0.0625, -0.453125},\n'''}
		
	]
	
	actions=migrations['0.12.35-0.13.0']()
	actions.append(simple_renaming(simple))
	actions.append(update_prototype(hard))
	
	
	return actions

migrations['0.12.35-0.13.3']=_
json_updates['0.12.35-0.13.3']=json_updates['0.12.35-0.13.0']

main()
